package info.textgrid.lab.collatex.ui.parts;

import info.textgrid.lab.collatex.model.CollationResults;

import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.IPage;
import org.eclipse.ui.part.PageBook;
import org.eclipse.ui.part.PageBookView;

/**
 * Abstract base class of views that display {@link CollationResults}.
 * 
 * Clients should implement {@link #createResultPage()} to create an appropriate
 * descendant of {@link AbstractResultPage}, and this should be enough.
 * 
 * This class cares for creating a {@linkplain DefaultResultPage default page},
 * appropriately initializing the result page and its control, and associating
 * the {@link AbstractResultPage} with the matching {@link CollationSetEditor}.
 */
public abstract class AbstractResultView extends PageBookView {

	public AbstractResultView() {
		super();
	}

	@Override
	protected IPage createDefaultPage(final PageBook book) {
		final DefaultResultPage defaultResultPage = new DefaultResultPage();
		initPage(defaultResultPage);
		defaultResultPage.createControl(book);
		return defaultResultPage;
	}

	/**
	 * @return a fresh {@link AbstractResultPage} implementation that will then
	 *         be initialized.
	 */
	protected abstract AbstractResultPage createResultPage();

	@Override
	protected PageRec doCreatePage(final IWorkbenchPart part) {
		final AbstractResultPage page = createResultPage();
		final PageRec rec = new PageRec(part, page);
		page.setEditor((CollationSetEditor) part);
		initPage(page);
		page.createControl(getPageBook());

		return rec;
	}

	@Override
	protected void doDestroyPage(final IWorkbenchPart part, final PageRec pageRecord) {
		pageRecord.page.dispose();
	}

	@Override
	protected IWorkbenchPart getBootstrapPart() {
		final IEditorPart activeEditor = getSite().getPage().getActiveEditor();
		if (activeEditor != null && isImportant(activeEditor))
			return activeEditor;

		final IEditorReference[] editorReferences = getSite().getPage()
				.getEditorReferences();
		for (final IEditorReference reference : editorReferences) {
			final IEditorPart editor = reference.getEditor(false);
			if (editor != null && isImportant(editor))
				return editor;
		}
		return null;
	}

	@Override
	protected boolean isImportant(final IWorkbenchPart part) {
		return part instanceof CollationSetEditor;
	}

}